//
//              VIEWSKEL.CPP
//              ============
//
// The skeleton application of VpeView.EXE
//
// This source code may be used and modified freely as long as the name
// of the resulting application is NOT "VPEVIEW.EXE".
//
// Notes:
// ------
// This source code uses the class "FileDialog" which is a class
// of IDEAL Software's internal application framework. This source code
// will not compile until you replace the use of "FileDialog" with
// an adequate class or function call.
//
// The call to VpeLicense() should be filled up with your license key.
//
// Uncomment the "#define PROFESSIONAL", if you compile with the
// VPE Professional or any higher edition.
//
//
// Format of the commandline:
// ==========================
// vpeview [options] filename [output-device]
//
// options:
//   -p = print only to default device
//   -t = print only to device specified in [output-device]
//
// output-device:
//   "|" + name of the output device
//
// Examples:
//    vpeview test.vpe           opens the preview with the specified file
//    vpeview -p test.vpe        prints the specified file to the default output device without preview
//    vpeview -t test.vpe|HP5P   prints the specified file to the output device named "HP5P" without preview
//
//
// For informations on setting up the registry to associate the ".VPE" file extension with the viewer,
// please consult the manual or help file named "Programmer's Manual", section "Programming Techniques",
// subsection "VPE View - the Document Viewer".
//
//
// Copyright  1993 - 2000 IDEAL Software. All rights reserved.
//
//

#define STRICT
#include <windows.h>
#include <windowsx.h>

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <io.h>
#include <math.h>
#include <time.h>
#include <ctype.h>

#include "colors.h"
#include "vpecomon.h"
#include "vpiface.h"

// ========================================================================
//                              Defines
// ======================================================================== 
// #define PROFESSIONAL	// uncomment, if you compile with the VPE Professional or any higher edition

#ifdef PROFESSIONAL
   #include "vppiface.cpp"
#endif


// ========================================================================
//                              Prototypes
// ======================================================================== 



// ========================================================================
//                              Globals
// ======================================================================== 
char szAppName [] = "VPE View";
BOOL PrintOnly = FALSE;
char *FileName = NULL;
char *DeviceName = NULL;
long hDoc = 0;

char *DocFilter[] = // Filter fr FileDialog
{
   "VPE Document (*.VPE)",  "*.vpe",
   "\0", "\0"
};


// ========================================================================
//                              DoRegistry
// ======================================================================== 
#ifdef __NT__
void DoRegistry(BOOL read, WINDOWPLACEMENT &wp)
{
   int ret;
   char value[512];
   char *key = "SOFTWARE\\IDEAL Software\\VPE View\\Settings";
   char *value_name = "MainFrame-Placement";
   DWORD value_type, value_size = sizeof(value);
   HKEY hkResult, hkResult2;

   if (read)
   {
      // set default values:
      wp.flags = 0;
      wp.showCmd = SW_SHOW;
      wp.ptMinPosition.x = -1;
      wp.ptMinPosition.y = -1;
      wp.ptMaxPosition.x = -1;
      wp.ptMaxPosition.y = -1;
      wp.rcNormalPosition.left   = 0;
      wp.rcNormalPosition.top    = 0;
      wp.rcNormalPosition.right  = 520;
      wp.rcNormalPosition.bottom = 390;

      ret = RegOpenKeyEx(HKEY_CURRENT_USER, key, NULL, KEY_ALL_ACCESS, &hkResult);
      if (ret != ERROR_SUCCESS)
         return;

      ret = RegQueryValueEx(hkResult, value_name, NULL, &value_type, (BYTE *)value, &value_size);
      if (ret == ERROR_SUCCESS)
         sscanf(value, "%d,%d,%d,%d,%d,%d,%d,%d,%d,%d", 
                &wp.flags,
                &wp.showCmd,
                &wp.ptMinPosition.x,
                &wp.ptMinPosition.y,
                &wp.ptMaxPosition.x,
                &wp.ptMaxPosition.y,
                &wp.rcNormalPosition.left,
                &wp.rcNormalPosition.top,
                &wp.rcNormalPosition.right,
                &wp.rcNormalPosition.bottom);
   }
   else
   {
      ret = RegOpenKeyEx(HKEY_CURRENT_USER, NULL, NULL, KEY_ALL_ACCESS, &hkResult);
      if (ret != ERROR_SUCCESS)
         return;
         
      RegCreateKey(hkResult, key, &hkResult2);
      wsprintf(value, "%d,%d,%d,%d,%d,%d,%d,%d,%d,%d", 
               wp.flags,
               wp.showCmd,
               wp.ptMinPosition.x,
               wp.ptMinPosition.y,
               wp.ptMaxPosition.x,
               wp.ptMaxPosition.y,
               wp.rcNormalPosition.left,
               wp.rcNormalPosition.top,
               wp.rcNormalPosition.right,
               wp.rcNormalPosition.bottom);
      RegSetValueEx(hkResult2, value_name, NULL, REG_SZ, (BYTE *)value, strlen(value));

      RegCloseKey(hkResult2);
   }
   
   RegCloseKey(hkResult);
}
#endif



// ========================================================================
//                              View
// ======================================================================== 
void View(HWND hwnd)
{
   char pure_filename[_MAX_PATH];
   char s[_MAX_PATH + 32];
   FileDialog FDialog(NULL, DocFilter);

   if (!FileName)
   {
      if (!FDialog.GetOpenName(hwnd))
         goto err_exit;

      FileName = strdup(FDialog.Fname);
   }

   if (access(FileName, F_OK) != 0)     // Datei nicht vorhanden
   {
      MessageBox(hwnd, "File not found!", szAppName, MB_OK | MB_ICONSTOP);
      goto err_exit;
   }

   hDoc = VpeOpenDocFile(hwnd, FileName, szAppName, VPE_EMBEDDED | VPE_DOCFILE_READONLY | VPE_GRIDBUTTON | VPE_FIXED_MESSAGES);

   if (!hDoc)
   {
      MessageBox(hwnd, "Error opening file!\n\n"
                       "Possible reasons:\n"
                       "- Incompatible Document Version\n"
                       "- Access denied", szAppName, MB_OK | MB_ICONSTOP);
      goto err_exit;
   }

   // VpeLicense(); // insert your license key here

  #ifdef PROFESSIONAL
   VpeEnableClickEvents(hDoc, FALSE);
  #endif
      
   _splitpath(FileName, NULL, NULL, pure_filename, NULL);
   wsprintf(s, "%s - %s", pure_filename, szAppName);
   VpeSetDevJobName(hDoc, s);
   if (PrintOnly)
   {
      if (DeviceName)
         VpeSetDevice(hDoc, DeviceName);
      VpePrintDoc(hDoc, FALSE);
      VpeCloseDoc(hDoc);
   }
   else
      VpePreviewDoc(hDoc, NULL, VPE_SHOW_NORMAL);
      
   return;
   
err_exit:
   PostMessage(hwnd, WM_DESTROY, 0, 0);
   return;
}




// ========================================================================
//                              WndProc
// ========================================================================
long FAR PASCAL _export WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
   switch (message)
   {
    case  WM_CREATE:
       PostMessage(hwnd, WM_USER, 0, 0);
       break;
       
    case WM_USER:
       View(hwnd);
       return 0;

    case WM_SIZE:
       if (hDoc)
          MoveWindow(VpeGetWindowHandle(hDoc), 0, 0, LOWORD(lParam), HIWORD(lParam), TRUE);
       return 0;

    case WM_KEYDOWN:
       if (hDoc)
          SendMessage(VpeGetWindowHandle(hDoc), WM_KEYDOWN, wParam, lParam);
       return 0;

    case WM_SETFOCUS:
       if (hDoc)
          SendMessage(VpeGetWindowHandle(hDoc), WM_SETFOCUS, wParam, lParam);
       return 0;

    case WM_KILLFOCUS:
       if (hDoc)
          SendMessage(VpeGetWindowHandle(hDoc), WM_KILLFOCUS, wParam, lParam);
       return 0;

    case VPE_DESTROYWINDOW:
     	 hDoc = 0;
     	 PostMessage(hwnd, WM_CLOSE, 0, 0);
       break;
       
    case WM_DESTROY:
      #ifdef __NT__
       if (!PrintOnly)
       {
         WINDOWPLACEMENT wp;

         wp.length = sizeof(WINDOWPLACEMENT);
         GetWindowPlacement(hwnd, &wp);
         DoRegistry(FALSE, wp);
       }
      #endif
       if (hDoc && VpeCloseDoc(hDoc))
          hDoc = 0;
       PostQuitMessage(0);
       return 0;
   }

   return DefWindowProc(hwnd, message, wParam, lParam);
}


void eliminate_qmarks(char *s)
{
   // Eliminate Quotation marks "":
   int len = strlen(s);
   for (int i = 0; i < len; i++)
   {
      if (s[i] == '"')
      {
         for (int n = i+1; n <= len; n++)
            s[n-1] = s[n];
      }
   }
}


void skip_blanks(char **cmdline)
{
   while (**cmdline && **cmdline == ' ')
      (*cmdline)++;
}


// ========================================================================
//                              WinMain
// ========================================================================
#ifdef __NT__
   int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR cmdline, int )
#else
   int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR cmdline, int show_cmd)
#endif
{
   MSG         msg;
   HWND        hwnd;
   WNDCLASS    wndclass;

   // Parse cmdline:
   // ==============
   if (strlen(cmdline))
   {
      eliminate_qmarks(cmdline);
      skip_blanks(&cmdline);
      if (*cmdline && (*cmdline == '//' || *cmdline == '-') && tolower(cmdline[1]) == 'p')
      {
         PrintOnly = TRUE;
         cmdline += 2;
         skip_blanks(&cmdline);

         if (*cmdline)
            FileName = cmdline;
      }
      else if (*cmdline && (*cmdline == '//' || *cmdline == '-') && tolower(cmdline[1]) == 't')
      {
         PrintOnly = TRUE;
         cmdline += 2;
         skip_blanks(&cmdline);

         if (*cmdline)
            FileName = cmdline;
            
         char *p = strchr(cmdline, '|');
         if (p)
         {
            DeviceName = p;
            *DeviceName++ = 0;
         }
      }
      else if (*cmdline)
         FileName = cmdline;
   }

   if (!hPrevInstance)
   {
      wndclass.style         = 0;
      wndclass.lpfnWndProc   = WndProc;
      wndclass.cbClsExtra    = 0;
      wndclass.cbWndExtra    = 0;
      wndclass.hInstance     = hInstance;
      wndclass.hIcon         = LoadIcon(hInstance, "APP_ICON");
      wndclass.hCursor       = LoadCursor(NULL, IDC_ARROW);
      wndclass.hbrBackground = GetStockBrush(WHITE_BRUSH);
      wndclass.lpszMenuName  = "MAINMENU";
      wndclass.lpszClassName = szAppName;
      RegisterClass(&wndclass);
   }

   hwnd = CreateWindow(szAppName, szAppName,
                       WS_OVERLAPPEDWINDOW,
                       CW_USEDEFAULT, CW_USEDEFAULT,
                       CW_USEDEFAULT, CW_USEDEFAULT,
                       NULL, NULL, hInstance, NULL);

   if (!PrintOnly)
   {
     #ifdef __NT__
      WINDOWPLACEMENT wp;
      wp.length = sizeof(WINDOWPLACEMENT);
      DoRegistry(TRUE, wp);
      SetWindowPlacement(hwnd, &wp);
      ShowWindow(hwnd, wp.showCmd);
     #else
      ShowWindow(hwnd, show_cmd);
      UpdateWindow(hwnd);
     #endif
   }

   while (GetMessage(&msg, NULL, 0, 0))
   {
      TranslateMessage(&msg);
      DispatchMessage(&msg);
   }

   return msg.wParam;
}

